<?php

declare(strict_types=1);

namespace FiloBlu\RuleDayFromBirthday\Model;

use FiloBlu\Core\Framework\Model\AbstractSqlRepository;
use FiloBlu\Core\Framework\Model\RepositoryItem;
use FiloBlu\RuleDayFromBirthday\Api\CustomCouponRepositoryInterface;
use FiloBlu\RuleDayFromBirthday\Api\Data\{CustomCouponInterfaceFactory, CustomCouponInterface};
use Magento\Framework\Api\Search\SearchCriteriaInterfaceFactory;
use Magento\Framework\Api\SearchResultsInterfaceFactory;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NoSuchEntityException;
use Zend_Db;

/**
 * @class CustomCouponRepository
 * @package FiloBlu\RuleDayFromBirthday\Model
 */
class CustomCouponRepository extends AbstractSqlRepository implements CustomCouponRepositoryInterface
{

    /** @var string */
    public const TABLE_NAME = 'filoblu_sales_rule_coupon';

    /**
     * @var CustomCouponInterfaceFactory
     */
    private $customCouponFactory;

    /**
     * @param ResourceConnection $resourceConnection
     * @param SearchCriteriaInterfaceFactory $searchCriteriaFactory
     * @param SearchResultsInterfaceFactory $searchResultsFactory
     * @param CustomCouponInterfaceFactory $customCouponFactory
     */
    public function __construct(
        ResourceConnection             $resourceConnection,
        SearchCriteriaInterfaceFactory $searchCriteriaFactory,
        SearchResultsInterfaceFactory  $searchResultsFactory,
        CustomCouponInterfaceFactory   $customCouponFactory
    ) {
        $this->customCouponFactory = $customCouponFactory;
        $this->resourceConnection = $resourceConnection;
        parent::__construct($resourceConnection, $searchCriteriaFactory, $searchResultsFactory);
    }

    public function getById($id): RepositoryItem
    {
        try {
            return parent::getById($id);
        } catch (NoSuchEntityException $exception) {
            throw new NoSuchEntityException(__("Could not find a CustomCoupon with id = $id"));
        }
    }

    /**
     * @return string
     */
    public function getTable(): string
    {
        return self::TABLE_NAME;
    }

    public function getIdFieldName(): string
    {
        return CustomCouponInterface::ROW_ID;
    }

    /**
     * @param array $row
     * @return CustomCouponInterface
     */
    public function fromRow(array $row): CustomCouponInterface
    {
        return $this->customCouponFactory->create(['data' => $row]);
    }

    /**
     * @param CustomCouponInterface $customCoupon
     * @return CustomCouponInterface
     */
    public function save(CustomCouponInterface $customCoupon): CustomCouponInterface
    {
        $connection = $this->resourceConnection->getConnection();
        $table = $connection->getTableName(self::TABLE_NAME);

        $bindings = [
            CustomCouponInterface::ENTITY_ID => $customCoupon->getEntityId(),
            CustomCouponInterface::RULE_ID => $customCoupon->getRuleId(),
            CustomCouponInterface::EXPIRATION_DAYS => $customCoupon->getExpirationDays(),
            CustomCouponInterface::TYPE => $customCoupon->getType(),
            CustomCouponInterface::PREFIX => $customCoupon->getPrefix(),
            CustomCouponInterface::SUFFIX => $customCoupon->getSuffix(),
            CustomCouponInterface::LENGTH => $customCoupon->getLength(),
            CustomCouponInterface::FORMAT => $customCoupon->getFormat(),
            CustomCouponInterface::DASH => $customCoupon->getDash()
        ];

        $connection->insertOnDuplicate(
            $table,
            $bindings,
            [
                CustomCouponInterface::ENTITY_ID,
                CustomCouponInterface::RULE_ID,
                CustomCouponInterface::EXPIRATION_DAYS,
                CustomCouponInterface::TYPE,
                CustomCouponInterface::PREFIX,
                CustomCouponInterface::SUFFIX,
                CustomCouponInterface::LENGTH,
                CustomCouponInterface::FORMAT,
                CustomCouponInterface::DASH
            ]);

        $customCoupon->setEntityId((int)$connection->lastInsertId());

        return $customCoupon;
    }

    /**
     * @param $value
     * @param string $column
     * @return CustomCouponInterface
     * @throws NoSuchEntityException
     */
    public function getByKey($value, string $column): CustomCouponInterface
    {
        $connection = $this->resourceConnection->getConnection();
        $table = $connection->getTableName($this->getTable());

        $select = $connection->select()->from($table)->where(sprintf('%s = :%s', $column, $column));

        $item = $connection->fetchRow($select, [$column => $value], Zend_Db::FETCH_ASSOC);

        if (empty($item)) {
            throw new NoSuchEntityException(__(sprintf("Could not find Custom Coupon with %s = %s", $column, (string)$value)));
        }

        return $this->fromRow($item);
    }
}
